use std::path::PathBuf;
use tim_tool::logic::project::Job;
use tool_helper::{Error, Output};

pub struct CppHeader {
    file_out: Output,
}

impl CppHeader {
    pub fn new(file_path: &Option<PathBuf>) -> Result<CppHeader, Error> {
        let mut file_out = tool_helper::open_output(file_path)?;

        writeln!(file_out, "#pragma once")?;
        writeln!(file_out, "#include <PSX/File/cd_file_types.hpp>")?;
        writeln!(file_out, "")?;

        Ok(CppHeader{file_out})
    }

    pub fn push_job(&mut self, job: &Job, size: (u16, u16)) -> Result<(), Error> {
        let name             = job.name.replace('.', "_");
        let (clut_x, clut_y) = if let Some(palette_rect) = &job.palette_rect {(palette_rect.pos.x, palette_rect.pos.y)} else {(0, 0)};

        writeln!(self.file_out, "namespace {} {{", name)?;
            writeln!(self.file_out, "\tusing namespace JabyEngine;")?;
            writeln!(self.file_out, "")?;
            writeln!(self.file_out, "\tstatic constexpr auto tim  = SimpleTIM::create({}, {}, {}, {});", job.image_pos.x, job.image_pos.y, clut_x, clut_y)?;
            writeln!(self.file_out, "\tstatic constexpr auto size = GPU::SizeI16::create({}, {});", size.0, size.1)?;
        writeln!(self.file_out, "}}")?;

        Ok(())
    }
}

impl Drop for CppHeader {
    fn drop(&mut self) {
        
    }
}